from functools import wraps
from diskcache import Cache
from loguru import logger
import hashlib
from .core import DEFAULT_CACHE_DIR

def disk_cache(cache_dir=DEFAULT_CACHE_DIR, ttl=60*60*24):
    """磁盘缓存装饰器
    
    Args:
        cache_dir: 缓存目录
        ttl: 缓存过期时间（秒），默认24小时
    """
    cache = Cache(cache_dir)
    
    def _decorator(func):
        def make_key(args, kwargs):
            """生成缓存键"""
            key_parts = [
                func.__name__,
                str(args),
                str(sorted(kwargs.items()))
            ]
            key_str = '-'.join(key_parts)
            return hashlib.md5(key_str.encode('utf-8')).hexdigest()
        
        @wraps(func)
        def wrapper(*args, **kwargs):
            try:
                key = make_key(args, kwargs)
                result = cache.get(key)
                
                if result is not None:
                    logger.debug("磁盘缓存命中: func={}, key={}", func.__name__, key)
                    return result
                
                result = func(*args, **kwargs)
                cache.set(key, result, expire=ttl)
                logger.debug("缓存未命中: func={}, key={}", func.__name__, key)
                return result
                
            except Exception as e:
                logger.error("缓存操作失败: {}", str(e))
                return func(*args, **kwargs)
        
        wrapper.clear_cache = cache.clear
        return wrapper
    
    return _decorator